namespace SpreadsheetGearExplorer
{
    partial class PerformanceSample
    {
        private void RunSample(
            SpreadsheetGear.Windows.Forms.WorkbookView workbookView)
        {
            // Do a little benchmark to get the code JITed.
            RunBenchmark(workbookView, 10); 

            // Refresh the WorkbookView.
            workbookView.Refresh();

            // Do the real benchmark which creates 50000 rows by 4 columns.
            RunBenchmark(workbookView, 50000);
        }

        private void RunBenchmark(
            SpreadsheetGear.Windows.Forms.WorkbookView workbookView, int rowCount)
        {
            // NOTE: Must acquire a workbook set lock.
            workbookView.GetLock();
            try
            {
                // Create a new workbook and assign it to the WorkbookView.
                SpreadsheetGear.IWorkbook workbook =
                    SpreadsheetGear.Factory.GetWorkbook(System.Globalization.CultureInfo.CurrentCulture);
                workbookView.ActiveWorkbook = workbook;

                // iter == 0 is the slow way.
                // iter == 1 is the fast way.
                for (int iter = 0; iter < 2; iter++)
                {
                    // Ger a worksheet and a working range.
                    SpreadsheetGear.IWorksheet worksheet = null;
                    switch (iter)
                    {
                        case 0:
                            worksheet = workbook.Worksheets[0];
                            worksheet.Name = "The Slow Way";
                            break;
                        case 1:
                            worksheet = workbook.Worksheets.Add();
                            worksheet.Name = "The Fast Way";
                            break;
                    }
                    SpreadsheetGear.IRange cells = worksheet.Cells;
                    SpreadsheetGear.IRange range = cells[3, 0, 3 + rowCount - 1, 3];

                    // Must call System.GC.Collect() for consistent timing.
                    System.GC.Collect(); 
                    
                    // Start the timer.
                    System.Diagnostics.Stopwatch stopWatch = System.Diagnostics.Stopwatch.StartNew();

                    // Do the work.
                    switch (iter)
                    {

                        case 0:
                            RunSlow(range);
                            cells["A1"].Formula = "The slow way took 0.80 seconds on an overclocked Intel Core i7-980X, Release Build, Run Without Debugging.";
                            break;
                        case 1:
                            RunFast(range);
                            cells["A1"].Formula = "The fast way took 0.03 seconds on an overclocked Intel Core i7-980X, Release Build, Run Without Debugging.";
                            break;
                    }

                    // Save the elapsed time in cell A2.
                    cells["A2"].Formula = "Elapsed time: " + (stopWatch.Elapsed.TotalMilliseconds / 1000.0) + " seconds.";
                }
            }
            finally
            {
                // NOTE: Must release the workbook set lock.
                workbookView.ReleaseLock();
            }
        }

        void RunSlow(SpreadsheetGear.IRange range)
        {
            for (int row = 0; row < range.RowCount; row++)
            {
                // Set some text in column A, numbers in columns B and C and a formula in column D.
                range[row, 0].Value = "Row" + row;
                range[row, 1].Value = row;
                range[row, 2].Value = row * 2;
                range[row, 3].Formula = "=SUM(" + range.WorkbookSet.GetAddress(row + range.Row, 1, row + range.Row, 2,
                    0, 0, false, false, false, false, false, SpreadsheetGear.ReferenceStyle.A1) + ")";
                
                // Set a number format on the numeric cells.
                range[row, 1, row, 3].NumberFormat = "#,##0";
                
                // Format every other row with a gray background.
                if ((row & 1) == 0)
                    range[row, 0, row, 3].Interior.Color = SpreadsheetGear.Colors.LightGray;
            }
        }

        void RunFast(SpreadsheetGear.IRange range)
        {
            // Get the IValues interface.
            SpreadsheetGear.Advanced.Cells.IValues values =
                (SpreadsheetGear.Advanced.Cells.IValues)range.Worksheet;

            // Get the starting row and row count.
            int startRow = range.Row;
            int rowCount = range.RowCount;
            
            // Loop through - but set the formula and the format once after looping.
            for (int row = 0; row < rowCount; row++)
            {
                // Set some text in column A, numbers in columns B and C.
                values.SetText(row + startRow, 0, "Row" + row);
                values.SetNumber(row + startRow, 1, row);
                values.SetNumber(row + startRow, 2, row * 2);
            }

            // Set a formula in Column D all at once which is much faster since the
            // formula is only created once and only parsed once.
            range[0, 3, rowCount - 1, 3].Formula = "=SUM(" + range.WorkbookSet.GetAddress(startRow, 1, startRow, 2,
                0, 0, false, false, false, false, false, SpreadsheetGear.ReferenceStyle.A1) + ")";

            // Set a number format on the numeric cells - much faster than one row at a time.
            range[0, 1, rowCount - 1, 3].NumberFormat = "#,##0";
 
            // Format every other row with a gray background using one conditional format.
            SpreadsheetGear.IFormatCondition formatCondition = 
                range.FormatConditions.Add(SpreadsheetGear.FormatConditionType.Expression, 
                SpreadsheetGear.FormatConditionOperator.None, "=MOD(ROW(),2)=0", "");
            formatCondition.Interior.Color = SpreadsheetGear.Colors.LightGray;
        }
    }
}
