namespace SpreadsheetGearExplorer
{
    partial class ValidationSample
    {
        private void AddValidationWholeNumber(
            SpreadsheetGear.Windows.Forms.WorkbookView workbookView)
        {
            // NOTE: Must acquire a workbook set lock.
            workbookView.GetLock();
            try
            {
                // Get a reference to a cell.
                SpreadsheetGear.IRange cell =
                    workbookView.ActiveWorksheet.Cells["A1"];

                // Get a reference to the cell's validation.
                SpreadsheetGear.IValidation validation = cell.Validation;

                // Delete any existing validation.
                validation.Delete();

                // Add data validation which will force the user to
                // enter a whole number greater than zero.
                validation.Add(
                    SpreadsheetGear.ValidationType.WholeNumber,
                    SpreadsheetGear.ValidationAlertStyle.Stop,
                    SpreadsheetGear.ValidationOperator.Greater, "0", null);

                // Set the error message to display if validation fails.
                validation.ErrorMessage = "You must enter a whole number greater than zero!";

                // Set the input message to display when the cell is active.
                validation.InputMessage = "Enter a whole number greater than zero.";

                // Select the cell and give focus to the WorkbookView so
                // that the validation input message tooltip will display.
                cell.Select();
                workbookView.Focus();
            }
            finally
            {
                // NOTE: Must release the workbook set lock.
                workbookView.ReleaseLock();
            }
        }

        private void AddValidationBetween(
            SpreadsheetGear.Windows.Forms.WorkbookView workbookView)
        {
            // NOTE: Must acquire a workbook set lock.
            workbookView.GetLock();
            try
            {
                // Get a reference to a cell.
                SpreadsheetGear.IRange cell =
                    workbookView.ActiveWorksheet.Cells["B1"];

                // Get a reference to the cell's validation.
                SpreadsheetGear.IValidation validation = cell.Validation;

                // Delete any existing validation.
                validation.Delete();

                // Add data validation which will force the user to
                // enter a decimal number between 100 and 500.
                validation.Add(
                    SpreadsheetGear.ValidationType.Decimal,
                    SpreadsheetGear.ValidationAlertStyle.Stop,
                    SpreadsheetGear.ValidationOperator.Between, "100", "500");

                // Set the error message to display if validation fails.
                validation.ErrorMessage = "You must enter a number between 100 and 500!";

                // Set the input message to display when the cell is active.
                validation.InputMessage = "Enter a number between 100 and 500.";

                // Select the cell and give focus to the WorkbookView so
                // that the validation input message tooltip will display.
                cell.Select();
                workbookView.Focus();
            }
            finally
            {
                // NOTE: Must release the workbook set lock.
                workbookView.ReleaseLock();
            }
        }

        private void AddValidationListValues(
            SpreadsheetGear.Windows.Forms.WorkbookView workbookView)
        {
            // NOTE: Must acquire a workbook set lock.
            workbookView.GetLock();
            try
            {
                // Get a reference to a cell.
                SpreadsheetGear.IRange cell =
                    workbookView.ActiveWorksheet.Cells["C1"];

                // Add data validation which will provide a drop-down 
                // list of values to choose from.  Users will be warned 
                // if they try to enter a value that is not in the list.
                cell.Validation.Add(
                    SpreadsheetGear.ValidationType.List,
                    SpreadsheetGear.ValidationAlertStyle.Warning,
                    SpreadsheetGear.ValidationOperator.Default, "Red,Green,Blue", null);

                // Select the cell.
                cell.Select();
            }
            finally
            {
                // NOTE: Must release the workbook set lock.
                workbookView.ReleaseLock();
            }
        }

        private void AddValidationListLinked(
            SpreadsheetGear.Windows.Forms.WorkbookView workbookView)
        {
            // NOTE: Must acquire a workbook set lock.
            workbookView.GetLock();
            try
            {
                // Load sample data.
                workbookView.ActiveWorksheet.Cells["D3:D5"].Value =
                    new string[,] { { "Cold" }, { "Warm" }, { "Hot" } };

                // Get a reference to a cell.
                SpreadsheetGear.IRange cell =
                    workbookView.ActiveWorksheet.Cells["D1"];

                // Add data validation which will provide a drop-down 
                // list of values to choose from.  This list is generated
                // from the sample cell data.  Users will be informed 
                // if they try to enter a value that is not in the list.
                cell.Validation.Add(
                    SpreadsheetGear.ValidationType.List,
                    SpreadsheetGear.ValidationAlertStyle.Information,
                    SpreadsheetGear.ValidationOperator.Default, "=D3:D5", null);

                // Select the cell.
                cell.Select();
            }
            finally
            {
                // NOTE: Must release the workbook set lock.
                workbookView.ReleaseLock();
            }
        }

        private void AddValidationCustom(
            SpreadsheetGear.Windows.Forms.WorkbookView workbookView)
        {
            // NOTE: Must acquire a workbook set lock.
            workbookView.GetLock();
            try
            {
                // Load sample data.
                workbookView.ActiveWorksheet.Cells["F1"].Value = 100;

                // Get a reference to a cell.
                SpreadsheetGear.IRange cell =
                    workbookView.ActiveWorksheet.Cells["E1"];

                // Get a reference to the cell's validation.
                SpreadsheetGear.IValidation validation = cell.Validation;

                // Delete any existing validation.
                validation.Delete();

                // Add data validation which will force the user to enter a
                // number which is validated against the specified formula.
                validation.Add(
                    SpreadsheetGear.ValidationType.Custom,
                    SpreadsheetGear.ValidationAlertStyle.Stop,
                    SpreadsheetGear.ValidationOperator.Default, 
                    "=IF(E1>(F1*2),TRUE,FALSE)", null);

                // Set the error message to display if validation fails.
                validation.ErrorMessage = "You must enter a number greater than (F1 x 2)";

                // Set the input message to display when the cell is active.
                validation.InputMessage = "Enter a number greater than (F1 x 2).";

                // Select the cell and give focus to the WorkbookView so
                // that the validation input message tooltip will display.
                cell.Select();
                workbookView.Focus();
            }
            finally
            {
                // NOTE: Must release the workbook set lock.
                workbookView.ReleaseLock();
            }
        }
    }
}
