Partial Class PerformanceSample

    Private Sub RunSample(ByRef WorkbookView As SpreadsheetGear.Windows.Forms.WorkbookView)

        ' Do a little benchmark to get the code JITed.
        RunBenchmark(WorkbookView, 10)

        ' Refresh the WorkbookView.
        WorkbookView.Refresh()

        ' Do the real benchmark which creates 50000 rows by 4 columns.
        RunBenchmark(WorkbookView, 50000)

    End Sub

    Private Sub RunBenchmark(ByVal workbookView As SpreadsheetGear.Windows.Forms.WorkbookView, ByVal rowCount As Integer)

        ' NOTE: Must acquire a workbook set lock.
        workbookView.GetLock()
        Try
            ' Create a new workbook and assign it to the WorkbookView.
            Dim workbook As SpreadsheetGear.IWorkbook = _
                SpreadsheetGear.Factory.GetWorkbook(System.Globalization.CultureInfo.CurrentCulture)
            workbookView.ActiveWorkbook = workbook

            ' iter == 0 is the slow way.
            ' iter == 1 is the fast way.
            Dim iter As Integer
            For iter = 0 To 1
                ' Ger a worksheet and a working range.
                Dim worksheet As SpreadsheetGear.IWorksheet = Nothing
                Select Case iter
                    Case 0
                        worksheet = workbook.Worksheets(0)
                        worksheet.Name = "The Slow Way"
                    Case 1
                        worksheet = workbook.Worksheets.Add()
                        worksheet.Name = "The Fast Way"
                End Select
                Dim cells As SpreadsheetGear.IRange = worksheet.Cells
                Dim range As SpreadsheetGear.IRange = cells(3, 0, 3 + rowCount - 1, 3)

                ' Must call System.GC.Collect() for consistent timing.
                System.GC.Collect()

                ' Start the timer.
                Dim stopWatch As System.Diagnostics.Stopwatch = System.Diagnostics.Stopwatch.StartNew()

                ' Do the work.
                Select Case iter
                    Case 0
                        RunSlow(range)
                        cells("A1").Formula = "The slow way took 0.80 seconds on an overclocked Intel Core i7-980X, " + _
                                              "Release Build, Run Without Debugging."
                    Case 1
                        RunFast(range)
                        cells("A1").Formula = "The fast way took 0.03 seconds on an overclocked Intel Core i7-980X, " + _
                                              "Release Build, Run Without Debugging."
                End Select

                ' Save the elapsed time in cell A2.
                Dim elapsedTime As Double = stopWatch.Elapsed.TotalMilliseconds
                cells("A2").Formula = "Elapsed time: " + (elapsedTime / 1000.0).ToString() + " seconds."
            Next iter
        Finally
            ' NOTE: Must release the workbook set lock.
            workbookView.ReleaseLock()
        End Try

    End Sub

    Private Sub RunSlow(ByVal range As SpreadsheetGear.IRange)

        Dim row As Integer
        For row = 0 To range.RowCount - 1
            ' Set some text in column A, numbers in columns B and C and a formula in column D.
            range(row, 0).Value = "Row" + row.ToString()
            range(row, 1).Value = row
            range(row, 2).Value = row * 2
            range(row, 3).Formula = "=SUM(" + range.WorkbookSet.GetAddress( _
                row + range.Row, 1, row + range.Row, 2, 0, 0, False, False, False, False, False, _
                SpreadsheetGear.ReferenceStyle.A1) + ")"

            ' Set a number format on the numeric cells.
            range(row, 1, row, 3).NumberFormat = "#,##0"

            ' Format every other row with a gray background.
            If (row And 1) = 0 Then
                range(row, 0, row, 3).Interior.Color = SpreadsheetGear.Colors.LightGray
            End If
        Next row

    End Sub

    Private Sub RunFast(ByVal range As SpreadsheetGear.IRange)

        ' Get the IValues interface.
        Dim values As SpreadsheetGear.Advanced.Cells.IValues = _
            CType(range.Worksheet, SpreadsheetGear.Advanced.Cells.IValues)

        ' Get the starting row and row count.
        Dim startRow As Integer = range.Row
        Dim rowCount As Integer = range.RowCount

        ' Loop through - but set the formula and the format once after looping.
        Dim row As Integer
        For row = 0 To rowCount - 1
            ' Set some text in column A, numbers in columns B and C.
            values.SetText(row + startRow, 0, "Row" + row.ToString())
            values.SetNumber(row + startRow, 1, row)
            values.SetNumber(row + startRow, 2, row * 2)
        Next row

        ' Set a formula in Column D all at once which is much faster since the
        ' formula is only created once and only parsed once.
        range(0, 3, rowCount - 1, 3).Formula = "=SUM(" + range.WorkbookSet.GetAddress( _
            startRow, 1, startRow, 2, 0, 0, False, False, False, False, False, _
            SpreadsheetGear.ReferenceStyle.A1) + ")"

        ' Set a number format on the numeric cells - much faster than one row at a time.
        range(0, 1, rowCount - 1, 3).NumberFormat = "#,##0"

        ' Format every other row with a gray background using one conditional format.
        Dim formatCondition As SpreadsheetGear.IFormatCondition = range.FormatConditions.Add( _
            SpreadsheetGear.FormatConditionType.Expression, SpreadsheetGear.FormatConditionOperator.None, "=MOD(ROW(),2)=0", "")
        formatCondition.Interior.Color = SpreadsheetGear.Colors.LightGray

    End Sub

End Class
