Partial Class ThreadingSample

    Private Sub AddThread(ByRef WorkbookView As SpreadsheetGear.Windows.Forms.WorkbookView)

        Dim worker As MyBackgroundWorker

        ' Get starting row.
        Dim row As Integer = myBackgroundWorkers.Count + 1

        ' Acquire a workbook set lock.
        WorkbookView.GetLock()
        Try
            ' Get the range for this thread.
            Dim range As SpreadsheetGear.IRange = _
                WorkbookView.ActiveWorksheet.Cells(row, FirstColumnIndex, row, LastColumnIndex)
            range.Select()

            ' Create the background worker.
            worker = New MyBackgroundWorker(range)
        Finally
            ' Release the workbook set lock.
            WorkbookView.ReleaseLock()
        End Try

        ' Do this work after releasing the lock so that we hold the lock
        ' for as little time as possible.
        myBackgroundWorkers.Add(worker)

        ' Create a thread and give it a name.
        Dim thread As System.Threading.Thread = New System.Threading.Thread(AddressOf worker.Run)
        thread.Name = myBackgroundWorkers.Count.ToString()

        ' Make it a background thread so it will die when the 
        ' application is closed.
        thread.IsBackground = True

        ' Start it up.
        thread.Start()

    End Sub

    Private Sub RemoveThread(ByRef WorkbookView As SpreadsheetGear.Windows.Forms.WorkbookView)

        Dim count As Integer = myBackgroundWorkers.Count
        If count <> 0 Then
            ' Stop the last worker and remove it from the list.
            Dim worker As MyBackgroundWorker = myBackgroundWorkers(count - 1)
            myBackgroundWorkers.Remove(worker)
            worker.StopThread()

            ' Acquire a workbook set lock.
            WorkbookView.GetLock()
            Try
                ' Select the previous worker.
                WorkbookView.ActiveWorksheet.Cells( _
                    count - 1, FirstColumnIndex, count - 1, LastColumnIndex).Select()
            Finally
                ' Release the workbook set lock.
                WorkbookView.ReleaseLock()
            End Try
        Else
            System.Windows.Forms.MessageBox.Show( _
                "There are no threads to remove.", "SpreadsheetGear")
        End If

    End Sub

    Dim myBackgroundWorkers As System.Collections.Generic.List(Of MyBackgroundWorker) = _
        New System.Collections.Generic.List(Of MyBackgroundWorker)()

    Private Class MyBackgroundWorker

        ' Initialize a worker. Note that we do not pass a WorkbookView to the
        ' background worker thread due to the fact that WorkbookView properties
        ' and methods should only be accessed on the GUI thread which created them.
        Public Sub New(ByVal range As SpreadsheetGear.IRange)

            System.Diagnostics.Debug.Assert( _
                (range.RowCount = 1 And range.ColumnCount = LastColumnIndex - FirstColumnIndex + 1))
            Me.range = range
            threadStop = False
            totalCount = 0

            ' Make sure each pseudo-random number generator has unique seed.
            rand = New System.Random(range.Row)

            ' Set this to true to use advanced API for better
            ' performance or false to use IRange.Value.
            fastWay = True

        End Sub

        ' Do work until stopped.
        Public Sub Run()

            ' Set thread status to Running and create formula.
            SetThreadStatus("Running", True)

            ' Cache the advanced cells IValues interface and starting
            ' row / column of the range for use later.
            Dim values As SpreadsheetGear.Advanced.Cells.IValues = _
                CType(range.Worksheet, SpreadsheetGear.Advanced.Cells.IValues)
            Dim row As Integer = range.Row
            Dim col As Integer = range.Column

            ' Loop until StopThread() is called.
            While True

                ' Must enter a critical section to call Monitor.Wait.
                SyncLock Me
                    If Not threadStop Then
                        ' Wait between zero and 1000 milliseconds (0.0 and 1.0 seconds),
                        ' or until StopThread() is called.
                        System.Threading.Monitor.Wait(Me, rand.Next(0, 1000))
                    End If
                    If threadStop Then
                        Exit While
                    End If
                End SyncLock

                ' Get some random data - do as much work as we can before getting
                ' the lock so we hold the lock for the shortest possible time.
                Dim lastCount As Double = rand.Next(1, 100)
                totalCount += lastCount
                iterations += 1

                ' Acquire the workbook set lock.
                '
                ' Note that IWorkbookSet.BeginUpdate / EndUpdate will be faster
                ' in cases where more changes are made for each acquired lock.
                range.WorkbookSet.GetLock()
                Try
                    ' fastWay is initialized in the constructor. Setting values
                    ' with IValues instead of IRange.Value is significantly faster.
                    '
                    ' It is important to note that a system with a large number of
                    ' background updates to process should have few worker threads 
                    ' which do as much work as possible during each GetLock / ReleaseLock,
                    ' unlike this sample which has one thread for each event and calls
                    ' GetLock / ReleaseLock for each and every event. Handling as many
                    ' events as possible within one GetLock / ReleaseLock adds significant
                    ' complexity, but should prove to be significantly more scalable
                    ' because it will cut down on the number of thread context switches.
                    If fastWay Then
                        ' Place the values the fast way.
                        values.SetNumber(row, col + IterationsColumnIndex, iterations)
                        values.SetNumber(row, col + LastCountColumnIndex, lastCount)
                        values.SetNumber(row, col + TotalCountColumnIndex, totalCount)
                    Else
                        ' Place the values the normal but slow way.
                        range(0, IterationsColumnIndex).Value = iterations
                        range(0, LastCountColumnIndex).Value = lastCount
                        range(0, TotalCountColumnIndex).Value = totalCount
                    End If
                Finally
                    ' Release the workbook set lock.
                    range.WorkbookSet.ReleaseLock()
                End Try

            End While
            SetThreadStatus("Stopped", False)

        End Sub

        Private Sub SetThreadStatus(ByVal status As String, ByVal createAverageFormula As Boolean)

            ' Acquire a workbook set lock.
            range.WorkbookSet.GetLock()
            Try
                Dim threadStatusCell As SpreadsheetGear.IRange = range(0, ThreadStatusColumnIndex)
                threadStatusCell.Formula = "Thread: " + System.Threading.Thread.CurrentThread.Name + " " + status
                threadStatusCell.EntireColumn.AutoFit()
                If createAverageFormula Then
                    range(0, AverageCountColumnIndex).Formula = String.Format("={0}/{1}", _
                        range.WorkbookSet.GetAddress(range.Row, range.Column + TotalCountColumnIndex), _
                        range.WorkbookSet.GetAddress(range.Row, range.Column + IterationsColumnIndex))
                End If
            Finally
                ' Release the workbook set lock.
                range.WorkbookSet.ReleaseLock()
            End Try

        End Sub

        ' Causes the background thread to stop.
        Public Sub StopThread()

            ' Enter critical section.
            SyncLock Me
                ' Let the background thread know...
                threadStop = True

                ' Wake up the background thread if it is waiting in Monitor.Wait.
                System.Threading.Monitor.Pulse(Me)
            End SyncLock

        End Sub

        ' Some constants for column indexing into the worker range.
        Private Const ThreadStatusColumnIndex As Integer = 0
        Private Const IterationsColumnIndex As Integer = 1
        Private Const LastCountColumnIndex As Integer = 2
        Private Const TotalCountColumnIndex As Integer = 3
        Private Const AverageCountColumnIndex As Integer = 4

        Private range As SpreadsheetGear.IRange ' Stores the range this worker will work on.
        Private threadStop As Boolean           ' Set to true to stop the worker.
        Private fastWay As Boolean              ' Specifies whether to use IRange.Value or advanced API.
        Private totalCount As Double            ' Accumulates the total for this worker.
        Private iterations As Double            ' Accumulates the number of iterations for this worker.
        Private rand As System.Random           ' For generating pseudo-random data.
    End Class

    ' Some constants for column indexing into ActiveWorksheet.Cells.
    Private Const FirstColumnIndex As Integer = 0
    Private Const LastColumnIndex As Integer = 4
End Class
